#include "AccessAttributes.h"
#include "stdafx.h"

using namespace pml;

CAccessAttributes::CAccessAttributes()
{
    m_pEveryoneSID;
    m_pAdminSID = NULL;
    m_pACL = NULL;
    m_pSD = NULL;
}

CAccessAttributes::~CAccessAttributes()
{    
    if (m_pEveryoneSID) 
        FreeSid(m_pEveryoneSID);
    if (m_pAdminSID) 
        FreeSid(m_pAdminSID);
    if (m_pACL) 
        LocalFree(m_pACL);
    if (m_pSD) 
        LocalFree(m_pSD);
}

void CAccessAttributes::InitAttributes(ACCESS_MODE accessMode)
{
    DWORD dwRes;
    EXPLICIT_ACCESS ea[2];
    SID_IDENTIFIER_AUTHORITY SIDAuthWorld = SECURITY_WORLD_SID_AUTHORITY;
    SID_IDENTIFIER_AUTHORITY SIDAuthNT = SECURITY_NT_AUTHORITY;

    // Create a well-known SID for the Everyone group.
    if(!AllocateAndInitializeSid(&SIDAuthWorld, 1, SECURITY_WORLD_RID, 0, 0, 0, 0, 0, 0, 0, &m_pEveryoneSID))
    {
        throw std::runtime_error("InitAttributes: AllocateAndInitializeSid  everyone failed");
    }

    // Initialize an EXPLICIT_ACCESS structure for an ACE. The ACE will deny Everyone access.
    ZeroMemory(&ea, 2 * sizeof(EXPLICIT_ACCESS));
    ea[0].grfAccessPermissions = KEY_READ;
    ea[0].grfAccessMode = accessMode;
    ea[0].grfInheritance= NO_INHERITANCE;
    ea[0].Trustee.TrusteeForm = TRUSTEE_IS_SID;
    ea[0].Trustee.TrusteeType = TRUSTEE_IS_WELL_KNOWN_GROUP;
    ea[0].Trustee.ptstrName  = (LPTSTR) m_pEveryoneSID;

    // Create a SID for the BUILTIN\Administrators group.
    if(! AllocateAndInitializeSid(&SIDAuthNT, 2, SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, &m_pAdminSID)) 
    {
        throw std::runtime_error("InitAttributes: AllocateAndInitializeSid admin failed");
    }

    // Initialize an EXPLICIT_ACCESS structure for an ACE. The ACE will deny the Administrators group full access to the process.
    ea[1].grfAccessPermissions = KEY_ALL_ACCESS;
    ea[1].grfAccessMode = accessMode;
    ea[1].grfInheritance= NO_INHERITANCE;
    ea[1].Trustee.TrusteeForm = TRUSTEE_IS_SID;
    ea[1].Trustee.TrusteeType = TRUSTEE_IS_GROUP;
    ea[1].Trustee.ptstrName  = (LPTSTR) m_pAdminSID;

    // Create a new ACL that contains the new ACEs.
    dwRes = SetEntriesInAcl(2, ea, NULL, &m_pACL);
    if (ERROR_SUCCESS != dwRes) 
    {
        throw std::runtime_error("InitAttributes: SetEntriesInAcl failed");
    }

    // Initialize a security descriptor.  
    m_pSD = (PSECURITY_DESCRIPTOR) LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH); 
    if (NULL == m_pSD) 
    { 
        throw std::runtime_error("InitAttributes: LocalAlloc failed");
    } 

    if (!InitializeSecurityDescriptor(m_pSD, SECURITY_DESCRIPTOR_REVISION)) 
    {  
        throw std::runtime_error("InitAttributes: InitializeSecurityDescriptor failed"); 
    } 
 
    // Add the ACL to the security descriptor. 
    if (!SetSecurityDescriptorDacl(m_pSD, 
            TRUE,     // bDaclPresent flag   
            m_pACL, 
            FALSE))   // not a default DACL 
    {  
        throw std::runtime_error("InitAttributes: SetSecurityDescriptorDacl failed"); 
    } 

    // Initialize a security attributes structure.
    m_sa.nLength = sizeof (SECURITY_ATTRIBUTES);
    m_sa.lpSecurityDescriptor = m_pSD;
    m_sa.bInheritHandle = FALSE;
}

SECURITY_ATTRIBUTES CAccessAttributes::GetAttributes()
{
    return m_sa;
}

PSECURITY_DESCRIPTOR CAccessAttributes::GetSecurityDescriptor()
{
    return m_sa.lpSecurityDescriptor;
}