#ifndef C_S_OBSERVER_H_
#define C_S_OBSERVER_H_

#include <map>
#include <vector>
#include <Wtsapi32.h>
#include "ISObserver.h"
#include "ThreadForSession.h"
#include "HandleGuard.h"
#include "boost/thread.hpp"
#include "boost/shared_ptr.hpp"

namespace pml
{
    typedef boost::shared_ptr<boost::thread> ThreadPtr;

    class ThreadHolder
    {
        boost::shared_ptr<CThreadForSession> m_pThreadParams;
        ThreadPtr m_thread;

        bool m_bStopInDestructor;

    public:
        ThreadHolder(boost::shared_ptr<CThreadForSession> pThreadParams, ThreadPtr thread)
            : m_bStopInDestructor(false)
        {
        }
        ~ThreadHolder()
        {
            if (m_bStopInDestructor)
            {
                if(m_pThreadParams)
                    m_pThreadParams->Stop();
                if(m_thread)
                    m_thread->join();
            }
        }
        void SetStopEvent (bool stop) { m_bStopInDestructor = stop; }
    };

    class CSObserver : public ISObserver
    {
    public:
        CSObserver();
        ~CSObserver();

        // ISObserver methods
        void GotLogon(const std::wstring& userName, int sid);
        void GotLogoff(int sid);
        void PutProcessHandleIntoMap(CHandleGuard &hProcGuard, int sid);

    private:
        typedef std::map<int, HANDLE> ProcessHandlesMap;
        ProcessHandlesMap m_hProcessesPool;

        typedef std::map<int, ThreadHolder> ThreadsMap;
        ThreadsMap m_threadsPool;

        boost::mutex m_mutex;

    };

    std::wstring CreateCommandLineStr();
    int CreateProcessInSessionId (int sid, CHandleGuard &hProcessGuard, CHandleGuard &hTokenGuard, const std::wstring & commandLine);
    bool GetProcessToken(int sid, CHandleGuard &hTokenGuard);
    std::wstring GetTokenUser(HANDLE Token);
    DWORD GetTokenSessionId(HANDLE TokenHandle);   

} //nm
#endif // C_S_OBSERVER_H_