#ifndef W_EVENT_H_
#define W_EVENT_H_

namespace sm
{
    class WinNotificationEvent
    {
        HANDLE hEvent_;

        WinNotificationEvent(const WinNotificationEvent &);
        WinNotificationEvent&operator =(const WinNotificationEvent &);
    public:
        WinNotificationEvent(BOOL bManulaReset = TRUE)
        {
            hEvent_ = ::CreateEvent(NULL, bManulaReset, FALSE, NULL);
            if(hEvent_ == NULL)
                throw std::runtime_error("can't create event.");
        }
        WinNotificationEvent(const wchar_t * pName, BOOL bManulaReset = TRUE, BOOL state = FALSE)
        {
            hEvent_ = ::CreateEventW(NULL, bManulaReset, state, pName);
            if(hEvent_ == NULL)
                throw std::runtime_error("can't create event.");
        }
        WinNotificationEvent(HANDLE hEvent)
            : hEvent_(hEvent)
        {}
        HANDLE getHandle()const
        {
            CheckOwn();
            return hEvent_;
        }
        PVOID getPointer()const
        {
            CheckOwn();
            return hEvent_;
        }
        void set()
        {
            CheckOwn();
            if (!::SetEvent(hEvent_))
                throw std::runtime_error("can't set event.");

        }
        void reset()
        {
            CheckOwn();
            if (!::ResetEvent(hEvent_))
                throw std::runtime_error("can't reset event");
        }
        bool wait(DWORD dwTimeInMs = INFINITE)
        {
            CheckOwn();
            DWORD dwRes = WaitForSingleObject(hEvent_,dwTimeInMs);
            if (dwRes == WAIT_TIMEOUT)
                return false;
            if (dwRes == WAIT_OBJECT_0)
                return true;
            throw std::runtime_error("can't wait event");
        }
        bool readState() const
        {
            CheckOwn();
            DWORD status = ::WaitForSingleObject(hEvent_, 0);
            if(status == WAIT_OBJECT_0)
                return true;
            return false;
        }
        HANDLE release()
        {
            HANDLE hEventCopy = hEvent_;
            hEvent_ = 0;
            return hEventCopy;
        }
        ~WinNotificationEvent()
        {
            if(hEvent_)
                CloseHandle(hEvent_);
        }

    private:
        void CheckOwn()const
        {
            if(hEvent_ == 0)
                throw std::exception(__FUNCTION__" Handle was released.");
        }
    };

} //nm

#endif // W_EVENT_H_