#include "StdAfx.h"
#include "SessionManager.h"

using namespace sm;

static const std::wstring g_fileName = L"\\SessMontService.log";

std::wstring GetLogFileName()
{
    wchar_t fileNameBuf[1024] = {0};
    if(!GetModuleFileNameW(0, fileNameBuf, sizeof(fileNameBuf) / sizeof(wchar_t) - 1))
    {
        throw std::runtime_error("GetLogFileName: GetModuleFileNameW failed");
    }
    std::wstring fileNameW(fileNameBuf);
    std::wstringstream logName;

    logName << strUtil::GetFilePath(fileNameW) <<g_fileName ;
    return logName.str();
}

void WriteLogInfo(const std::wstring &logFileName, const std::wstring & strInfo)
{
    std::wofstream file(logFileName.c_str(), std::ios_base::app | std::ios_base::out);
    file << strInfo.c_str();
    file.flush();
    file.close();
}

void CSessionManager::operator () ()
{
    try 
    {
        Common::CSessionOwner::GetActiveSessions(m_SessionsPool);
        for(SessionsSet::iterator it = m_SessionsPool.begin(); it != m_SessionsPool.end(); ++it)
        {

            std::wstringstream infoStr;
            infoStr << L"Session is active, user name: " << (*it).GetFullUserName().c_str() << L", sid = " << (*it).GetSid() << std::endl;
            WriteLogInfo(GetLogFileName(), infoStr.str());
            m_observer->GotLogon( (*it).GetFullUserName().c_str(), (*it).GetSid());

        }
    }
    catch(const std::exception &)
    {
        //write to log
    }

    for(;;)
    {
        DWORD dwEventsFlag(0);
        if(WTSWaitSystemEvent(WTS_CURRENT_SERVER_HANDLE, 
            WTS_EVENT_LOGOFF | WTS_EVENT_LOGON | WTS_EVENT_CONNECT | WTS_EVENT_DISCONNECT
            , &dwEventsFlag))
        {
            if( (WTS_EVENT_LOGON & dwEventsFlag) == WTS_EVENT_LOGON || (WTS_EVENT_LOGOFF & dwEventsFlag) == WTS_EVENT_LOGOFF ||
                (WTS_EVENT_CONNECT & dwEventsFlag) == WTS_EVENT_CONNECT || (WTS_EVENT_DISCONNECT & dwEventsFlag) == WTS_EVENT_DISCONNECT)
            {
                try
                {
                    UpdateSessionsState();
                }
                catch(const std::exception & ex)
                {
                    std::wstringstream infoStr;
                    infoStr << L"UpdateSessionsState failed: " << ex.what() << std::endl;
                    WriteLogInfo(GetLogFileName(), infoStr.str());
                }
            }
            if (dwEventsFlag == WTS_EVENT_NONE)
            {
                return; // exit
            }
        }
        else
        {
            if(GetLastError() == ERROR_OPERATION_ABORTED)
            {
                //TODO: write to log 
                return;
            }
            else
            {
                //TODO: write to log 
                Sleep(100);
            }
        }
    }
}

void CSessionManager::GotSessionsSetGhanges(SessionsSet & addedSessions, SessionsSet & lostSessions)
{
    Common::CSessionOwner::GetActiveSessions(addedSessions);
    for(SessionsSet::iterator sessionIt= m_SessionsPool.begin(); sessionIt != m_SessionsPool.end(); ++sessionIt)
    {
        SessionsSet::iterator currentSessionsIt = addedSessions.find(*sessionIt);
        if(currentSessionsIt == addedSessions.end())
        {
            std::pair<SessionsSet::iterator, bool> res = lostSessions.insert(*sessionIt);
            if(!res.second)
            {
                throw std::runtime_error("CSessionManager::GotSessionsSetGhanges error: Cannot insert logged off session into pool");
            }
        }
        else
        {
            addedSessions.erase(*sessionIt);
        }
    }

    // add new sessions to session pool    
    for(SessionsSet::iterator it = addedSessions.begin(); it != addedSessions.end(); ++it)
    {
        std::pair<SessionsSet::iterator, bool> res = m_SessionsPool.insert(*it);
        if(!res.second)
        {
            throw std::runtime_error("CSessionManager::GotSessionsSetGhanges error: Cannot insert logged off session into pool");
        }
    }

    // erase lost sessions from session pool
    for(SessionsSet::iterator it = lostSessions.begin(); it != lostSessions.end(); ++it)
    {
        m_SessionsPool.erase(*it);
    }
}

void CSessionManager::UpdateSessionsState()
{
    SessionsSet addedSessionsPool;
    SessionsSet lostSessionPool;
    GotSessionsSetGhanges(addedSessionsPool, lostSessionPool);
    for(SessionsSet::iterator it = addedSessionsPool.begin(); it != addedSessionsPool.end(); ++it)
    {
        std::wstringstream infoStr;
        infoStr << L"Session is logged on, user name: " << (*it).GetFullUserName().c_str() << L", sid: " << (*it).GetSid() << std::endl;
        WriteLogInfo(GetLogFileName(), infoStr.str());
        m_observer->GotLogon( (*it).GetFullUserName().c_str(), (*it).GetSid());
    }
    for(SessionsSet::iterator it = lostSessionPool.begin(); it != lostSessionPool.end(); ++it)
    {
        std::wstringstream infoStr;
        infoStr << L"Session is logged out, user name: " << (*it).GetFullUserName().c_str() << L", sid: " << (*it).GetSid() << std::endl;
        WriteLogInfo(GetLogFileName(), infoStr.str());
        m_observer->GotLogoff( (*it).GetSid() );
    }
}