#include "StdAfx.h"
#include "SessionOwner.h"

using namespace Common;

CSessionOwner::CSessionOwner(const int& sid)
:m_sid(sid)
{
}

CSessionOwner::~CSessionOwner()
{
}

void CSessionOwner::GetSessions(std::vector<CSessionOwner> & vecSessions)
{
    PWTS_SESSION_INFO pSessionInfo(NULL);
    DWORD count(0);
    if(!WTSEnumerateSessions( WTS_CURRENT_SERVER_HANDLE, 0, 1, &pSessionInfo, &count))
    {
        DWORD err = GetLastError();
        if (err == RPC_S_INVALID_BINDING)
            return;
        else
        {
            std::string errStr = "GetSessions failed";
            std::stringstream  errStrStream ;
            errStrStream << errStr << err;
            throw std::runtime_error(errStrStream.str());
        }
    }
    wts_resource<WTS_SESSION_INFO> wtsSessionInfo(pSessionInfo);
    for (DWORD i = 0; i < count; ++i)
    {
        int sid = wtsSessionInfo[i].SessionId;
        vecSessions.push_back(CSessionOwner(sid));
    }
}

void CSessionOwner::GetActiveSessions(std::set<CSessionOwner> & setSessions)
{
    std::vector<CSessionOwner> vecSessions;
    GetSessions(vecSessions);
    for(size_t i = 0; i < vecSessions.size(); ++i)
    {
        if(vecSessions.at(i).IsActive())
        {
            std::pair<std::set<CSessionOwner>::iterator, bool> res = setSessions.insert(vecSessions.at(i));
            if(!res.second)
            {
                throw std::runtime_error("Insert active session failed");
            }
        }
    }
}

int CSessionOwner::GetSid() const
{
    return m_sid;
}

bool CSessionOwner::IsActive() const
{
    DWORD bytesReturned(0);
    LPTSTR pBuf(NULL);
    if(!WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE, m_sid, WTSConnectState, &pBuf, &bytesReturned))
    {
        // this is not exceptional situation, the session may be closed at the moment, or close operation is pending on the session
        return false;
    }
    wts_resource<wchar_t> wtsPBuf(pBuf);
    int connectState = *(reinterpret_cast<int*> (wtsPBuf.get()));
    return (connectState == WTSActive);
}

double CSessionOwner::GetLogonTime() const
{
    DWORD bytesReturned(0);
    LPTSTR pBuf(NULL);
    if(!WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE, m_sid, WTSLogonTime, &pBuf, &bytesReturned))
    {
        throw std::runtime_error("GotLogonTime session time failed");
    }
    wts_resource<wchar_t> wtsPBuf(pBuf);
    return (*(reinterpret_cast<double*> (wtsPBuf.get())));
}

std::wstring CSessionOwner::GetSessionUserName() const
{
    DWORD bytesReturned(0);
    LPTSTR pBuf(NULL);
    if(!WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE, m_sid, WTSUserName, &pBuf, &bytesReturned))
    {
        throw std::runtime_error("GetSessionUserName session failed");
    }
    wts_resource<wchar_t> wtsPBuf(pBuf);
    std::wstring userName(wtsPBuf.get());
    return userName;
}

std::wstring CSessionOwner::GetDomainName() const
{
    DWORD bytesReturned(0);
    LPTSTR pBuf(NULL);
    if(!WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE, m_sid, WTSDomainName, &pBuf, &bytesReturned))
    {
        throw std::runtime_error("GetDomainName session failed");
    }
    wts_resource<wchar_t> wtsPBuf(pBuf);
    std::wstring domainName(wtsPBuf.get());
    return domainName;
}

std::wstring CSessionOwner::GetFullUserName() const
{
    try
    {
        return(GetDomainName() + L"\\" + GetSessionUserName());
    }
    catch(std::exception &)
    {
        throw std::runtime_error("GetFullUserName session failed");
    }
}